; VIC-20 assembly demonstration for Tower of Hanoi puzzle using recursion
; note that although the routine can handle many more, the display routine
; only handles up to 9 discs

;kernel routines 
CHAROUT = $ffd2
GETIN   = $ffe4
PLOT    = $fff0


; macros for stack push variable and pull variable actions

defm CPSH
     lda /1
     pha
endm

defm CPUL
     pla
     sta /1
endm

;program start 
*=$1001

        BYTE    $0E, $12, $0A, $00, $9E, $20, $28,  $34, $31, $36, $30, $29, $00, $00, $00

*=$1040
       jmp main ;entry point

;number of discs 
discs   byte 3  ;poke 4163,number of discs from BASIC and run

;recursive routine which stores its data on the system stack 

;tower params
counter byte 0
Pfrom   byte 0
Pto     byte 0
PAux    byte 0

tower  lda counter
       bne cont
       rts
 
cont   CPSH counter  ;put parameters for this call on stack
       CPSH Pfrom
       CPSH Pto
       CPSH PAux
       ldx counter
       dex
       stx counter
       ldy Pto    
       lda PAux
       sta Pto
       lda Pfrom
       sta Pfrom
       sty PAux
       jsr tower    ;recurse call 1 from = from, to = aux, aux = to
       CPUL PAux    ;recover parameters from stack 
       CPUL Pto
       CPUL Pfrom
       CPUL counter
       ;output move here move <counter> from <Pfrom> to <Pto>
       jsr outputmove

       ldx counter
       dex 
       stx counter
       ldy Pfrom
       lda Paux 
       sta Pfrom
       lda Pto
       sta Pto
       sty Paux
       jsr tower   ;recurse call 2 from = aux, to = to, aux = from
       rts

;routine to output the computed moves

row    byte 0   ;starting row for output 

outputmove clc
       ldx row
omb2   ldy #0
       jsr PLOT ;PLOT cursor
       inc row
       lda row
       cmp #23   ;at the bottom of the screen?
       bcc omf2
       jsr pause ;prompt for next screen page
       lda #147  ;Clear screen
       jsr CHAROUT 
       ldx #0
       stx row   ;reset to top of screen
       beq omb2  ;jump back for output on next screen
omf2   lda counter
       clc
       adc #$30
       sta message+5 ;slot for number of discs
       lda Pfrom
       sta message+12 ;slot for From pole
       lda Pto
       sta message+17 ;slot for To pole
       ldy #0
omb1   lda message,y
       beq omf1
       jsr CHAROUT  
       iny
       bne omb1
omf1   rts

message text "move   from   to  " 
        byte 0
       
pause  ldy #0
pb1    lda pmessage,y
       beq pb2
       jsr CHAROUT
       iny
       bne pb1
pb2    jsr GETIN ;wait for key press
       beq pb2
       rts

pmessage text "any key for next page"
         byte 0


main   lda #147  ;clear screen
       jsr CHAROUT
       lda #0    ;initialise program variables
       sta row

       ;start conditions
       lda discs
       sta counter ;number of discs
       lda #"a"    ;pole A holds all the discs
       sta Pfrom
       lda #"c"    ;pole C is the destination pole
       sta Pto
       lda #"b"    ;pole B is an intermediate pole
       sta PAux
       jsr tower   ;move the discs from A to C  
       rts
